"""Email handling and sending"""
import email.mime.text
import inspect
import socket
import subprocess
from email.utils import parseaddr

from jinja2 import Environment
from jinja2 import PackageLoader

import ocflib.account.validators
import ocflib.constants as constants
import ocflib.misc.validators as validators


jinja_mail_env = Environment(loader=PackageLoader('ocflib', ''))
jinja_mail_env.globals = {
    'mail_signature': constants.MAIL_SIGNATURE,
}


def email_for_user(username):
    """Return email for a user.

    Currently, just appends @ocf.berkeley.edu, but could eventually do
    something more complicated."""

    ocflib.account.validators.validate_username(username, check_exists=True)
    return '{}@ocf.berkeley.edu'.format(username)


def send_mail_user(user, subject, body, sender=constants.MAIL_FROM):
    """Send a plan-text mail message to a user."""
    send_mail(email_for_user(user), subject, body, sender=sender)


def send_mail(to, subject, body, sender=constants.MAIL_FROM):
    """Send a plain-text mail message.

    `body` should be a string with newlines, wrapped at about 80 characters."""

    if not validators.valid_email(parseaddr(sender)[1]):
        raise ValueError('Invalid sender address.')

    if not validators.valid_email(parseaddr(to)[1]):
        raise ValueError('Invalid recipient address.')

    msg = email.mime.text.MIMEText(body)

    msg['Subject'] = subject
    msg['From'] = sender
    msg['To'] = to

    # we send the message via sendmail, since we may one day prohibit traffic
    # to port 25 that doesn't go via the system mailserver
    p = subprocess.Popen((constants.SENDMAIL_PATH, '-t', '-oi'),
                         stdin=subprocess.PIPE)
    p.communicate(msg.as_string().encode('utf8'))


def send_problem_report(problem):
    """Send a problem report to OCF staff."""

    def format_frame(frame):
        _, filename, line, funcname, _, _ = frame
        return '{}:{} ({})'.format(filename, line, funcname)

    callstack = '\n        by '.join(map(format_frame, inspect.stack()))
    body = \
        """A problem was encountered and reported via ocflib:

{problem}

====
Hostname: {hostname}
Callstack:
    at {callstack}
""".format(problem=problem, hostname=socket.getfqdn(), callstack=callstack)

    send_mail(
        constants.MAIL_ROOT,
        '[ocflib] Problem report from ' + socket.getfqdn(),
        body,
        sender='ocflib <root@ocf.berkeley.edu>',
    )



import mock
import pytest

import ocflib.misc.shell
from ocflib.misc.shell import BG_CODES
from ocflib.misc.shell import FG_CODES
from ocflib.misc.shell import get_editor


def test_shell_quote():
    assert ocflib.misc.shell.escape_arg


@pytest.mark.parametrize('color', ocflib.misc.shell.COLORS)
def test_color_wrappers_with_tty(color):
    with mock.patch('sys.stdout.isatty', return_value=True):
        fore = getattr(ocflib.misc.shell, color)
        assert fore('hi') == fore('hi', tty_only=True) == FG_CODES[color] + 'hi' + FG_CODES['reset']

        bg = getattr(ocflib.misc.shell, 'bg_' + color)
        assert bg('hi') == bg('hi', tty_only=True) == BG_CODES[color] + 'hi' + BG_CODES['reset']


@pytest.mark.parametrize('color', ocflib.misc.shell.COLORS)
def test_color_wrappers_without_tty(color):
    with mock.patch('sys.stdout.isatty', return_value=False):
        fore = getattr(ocflib.misc.shell, color)
        assert fore('hi') == 'hi'

        bg = getattr(ocflib.misc.shell, 'bg_' + color)
        assert bg('hi') == 'hi'


@pytest.mark.parametrize('env,expected', [
    ({}, 'nano'),
    ({'VISUAL': 'vi'}, 'vi'),
    ({'VISUAL': 'vi', 'EDITOR': 'nano'}, 'vi'),
    ({'EDITOR': 'emacs'}, 'emacs'),
])
def test_get_editor(env, expected):
    with mock.patch('os.environ', env):
        assert get_editor() == expected



"""Misc validators for things like emails, domains, etc."""
import re

import dns
import dns.resolver

from ocflib.infra.net import OCF_DNS_RESOLVER


def host_exists(host):
    try:
        message = dns.message.make_query(host, dns.rdatatype.ANY)
    except dns.name.EmptyLabel:
        return False

    response = dns.query.udp(message, str(OCF_DNS_RESOLVER))
    return bool(response.answer)


def email_host_exists(email_addr):
    """Verifies that the host of the email address exists"""
    if '@' in email_addr:
        host = email_addr.rsplit('@', 1).pop()
        return host_exists(host)
    return False


def valid_email(email):
    """Check the email with naive regex and check for the domain's MX record.
    Returns True for valid email, False for bad email."""
    regex = r'^[a-z0-9._%\-+]+@([a-z0-9._%\-]+.[a-z]{2,})$'

    m = re.match(regex, email, re.IGNORECASE)
    if m:
        domain = m.group(1)
        try:
            # Check that the domain has MX record(s)
            return bool(dns.resolver.query(domain, 'MX'))
        except (dns.resolver.NoAnswer, dns.resolver.NXDOMAIN):
            pass
    return False
